/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/*
 * This file contains the functions that launch new windows such as
 * the help system, Rx, etc.
 *
 */

var _gLaunchLog = getLoggingMgr().getLogger("launch");
var _gLaunchBaseHelpURL = null;

/* XXX duplicated from help/content/contextHelp.js.  We do NOT want
   alwaysRaised attribute on the window, that's obnoxious! */
var helpFileURI;




var defaultOpenOptions = "chrome,all,close=yes";


function openHelp(topic, contentPack)
{
  helpFileURI = contentPack || helpFileURI;

  var topWindow = locateHelpWindow(helpFileURI);

  if ( topWindow ) {
    topWindow.focus();
    topWindow.displayTopic(topic);
  } else {
    const params = Components.classes["@mozilla.org/embedcomp/dialogparam;1"]
                             .createInstance(Components.interfaces.nsIDialogParamBlock);
    params.SetNumberStrings(2);
    params.SetString(0, helpFileURI);
    params.SetString(1, topic);
    var ww = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                         .getService(Components.interfaces.nsIWindowWatcher);
    ww.openWindow(null, "chrome://help/content/help.xul", "_blank", defaultOpenOptions, params);
  }
}

function setHelpFileURI(rdfURI)
{
  helpFileURI = rdfURI;
}

function locateHelpWindow(contentPack) {
    const windowManagerInterface = Components
        .classes['@mozilla.org/appshell/window-mediator;1'].getService()
        .QueryInterface(Components.interfaces.nsIWindowMediator);
    const iterator = windowManagerInterface.getEnumerator("mozilla:help");
    var topWindow = null;
    var aWindow;

    while (iterator.hasMoreElements()) {
        aWindow = iterator.getNext();
        if (aWindow.getHelpFileURI() == contentPack) {
            topWindow = aWindow;
        }
    }
    return topWindow;
}
/* end of contextHelp.js duplication */

function launch_MainHelp(page) {
    openHelp(page, 'chrome://komododoc/locale/komodohelp.rdf');
}

// XXX FIXME deprecate this list, we should just use the tags directly
var help_tag2page = {
    // Prefs panels.
    "Appearance Prefs panel": "appearance",
    "Code Intelligence Prefs panel": "code_intelligence_preferences",
    "Debugger Prefs panel": "debugger",
    "Connection Prefs panel": "debugger_proxy_preferences",
    "Debugger Proxy": "debugger_proxy",
    "Advanced Prefs panel": "advanced_debugger_preferences",
    "Directory Import Prefs panel": "directory_import",
    "Editor Prefs panel": "editor",
    "HTTP Inspector Prefs panel": "http_inspector_preferences",
    "Key Bindings Prefs panel": "key_bindings",
    "Indentation Prefs panel": "indentation",
    "Smart Editing Prefs panel": "smart_editing",
    "Save Options Prefs panel": "save_options",
    "Environment Prefs panel": "environment",
    "File Associations Prefs panel": "file_associations",
    "Fonts and Colors Prefs panel": "fonts_and_colors",
    "GUI Builder Prefs panel": "gui_builder_preferences",
    "Interactive Shell Prefs panel": "interactive_shell_preferences",
    "Internationalization Prefs panel": "internationalization_preferences",
    "Language Help Prefs panel": "language_help_preferences",
    "Languages Prefs panel": "language_configuration_preferences",
    "JavaScript Prefs panel": "javascript_configuration",
    "Perl Prefs panel": "perl_configuration",
    "PHP Prefs panel": "php_configuration",
    "Ruby Prefs panel": "ruby_configuration",
    "Python Prefs panel": "python_configuration",
    "XML Catalogs Prefs panel": "xml_catalogs",
    "Tcl Prefs panel": "tcl_configuration",
    //XXX The following two are Tcl-specific although its title doesn't
    //    suggest so. If the other languages grow in a similar way then
    //    will have to extend the Prefs panel name -> Help tag system.
    "HTML Prefs panel": "html_configuration",
    "Mapped URIs Prefs panel": "mapped_uris",
    "New Files Prefs panel": "new_files",
    "Printing Prefs panel": "printing",
    "Projects and Workspace Prefs panel": "projects_and_workspace",
    "Servers Prefs panel": "server_configuration",
    "Shared Support Prefs panel": "shared_support",
    "Source Code Control Prefs panel": "source_code_control_preferences",
    "CVS Prefs panel": "cvs_integration",
    "Perforce Prefs panel": "perforce_integration",
    "Subversion Prefs panel": "subversion_integration",
    "Web & Browser Prefs panel": "web_and_browser",
    "Windows Integration Prefs panel": "windows_integration",

    "Save Remotely As": "saving_files_remotely",
    "Open Remote File": "opening_remote_files",
    "Current File Settings": "file_properties_and_settings",
    "Find/Replace": "find_dialog",
    "Find in Files": "find_in_files_dialog",
    "Import from File System": "importing_from_a_file_system",
    "Snippet Properties": "snippet_options",
    "Command Properties": "run_command_properties",
    "URL Properties": "url_shortcut_options",
    "Directory Shortcut Properties": "open_shortcut_options",
    "Template Properties": "template_options",
    "Macro Properties": "macro_options",
    "Run Command": "run_command_properties",
    "Object Browser":"object_browser",
    "Debugging Options": "debugging_options",
    "New File": "creating_files",
    "Breakpoint/Spawnpoint Properties": "breakpoints_and_tcl_spawnpoints",
    "Custom Menu/Toolbar Properties": "custom_menu_and_toolbar_properties",

    // How about these:
    // (none)
    // Rejected for now (URLs not necessarily correct):
    // - "Breakpoints Tab": "komodo-doc-debugger.html#breakpoints_and_spawnpoints",
    // - "Projects Tab": "komodo-doc-project.html#Managing_Projects",
    // - "Toolbox Tab": "komodo-doc-toolbox.html",
    // - "Interactive Shell": "komodo-doc-intshell.html",
    
    // XXX FIXME these need toc entries to work
    "Proxies Prefs panel": "komodo-doc-prefs.html#Proxy",
    "Syntax Checking Prefs panel": "komodo-doc-prefs.html#Tcl_syntax",
    "Debugging Prefs panel": "komodo-doc-prefs.html#Tcl_debugging"
};

function launch_getHelpTag(tag) {
    if (tag in help_tag2page && help_tag2page[tag] != null) {
        return help_tag2page[tag];
    }
    return tag;
}
// Open Komodo's help to the appropriate page for the given "tag" name.
// A tag is some name that identifies a thing in Komodo on which there is
// specific help. The intention is to decouple the actual Komodo help
// URL and anchor from the association "thing". The mapping is hardcoded
// here.
function launch_MainHelpTag(tag) {
    var helpTag = launch_getHelpTag(tag)
    launch_MainHelp(helpTag);
}

function launch_TutorialProject(tutorial)
{
    _gLaunchLog.info("launch_TutorialProject");
    try {
        var response = dialog_okCancel(
            "Would you also like to open the project with which the tutorial works?",
            "OK");
        if (response != "OK") return;
        
        var koDirSvc = Components.classes["@activestate.com/koDirs;1"].
                getService(Components.interfaces.koIDirs);
        var osPathSvc = Components.classes["@activestate.com/koOsPath;1"].
                getService(Components.interfaces.koIOsPath);
        var sampleKpfPath = osPathSvc.joinlist(4,
                [koDirSvc.userDataDir, "samples", tutorial+"_tutorials", tutorial+"_tutorial.kpf"]);
        
        if (! osPathSvc.exists(sampleKpfPath)) {
            response = dialog_okCancel(
                "The sample project could not be found at '"+
                    sampleKpfPath+"'. Your Komodo samples directory "+
                    "may be corrupt. Would you like Komodo to "+
                    "re-generate the samples tree?",
                "Cancel");
            if (response == "OK") {
                var initSvc = Components.classes["@activestate.com/koInitService;1"].
                        getService(Components.interfaces.koIInitService);
                initSvc.installSamples(true);
            } else {
                return;
            }
        }

        var sampleKpfUrl = uriparse_pathToURI(sampleKpfPath);
        project_openProjectFromURL(sampleKpfUrl);
    } catch (ex) {
        _gLaunchLog.error("launch_TutorialProject error: "+ex);
    }
}

function launch_Tutorials(tutorial /* =null */)
{
    // Open the Tutorial in the help browser
    //    "tutorial" can optionally be used to specify a particular tutorial
    //        to start with. Currently valid values are "perl", "python",
    //        "php", "xslt", and null (to go to Tutorial home page).
    if (!tutorial) tutorial = 'showcase';
    var page = tutorial+"_tutorial";
    if (tutorial != 'showcase') {
        launch_TutorialProject(tutorial);
    }
    launch_MainHelp(page);
}


function launch_TipOfTheDay() {
    openWindowUniqueInstance("chrome://komodo/content/tipoftheday/tipoftheday.xul",
                             "komodo_tip",
                             "chrome,dialog,dependent,resizable");
}


function launch_LanguageHelp() {
    // Get the current document's language.
    var language = null;
    var view = gViewMgr.currentView;
    if (!view) view = gViewMgr.currentView;
    if (view != null) {
        language = view.document.subLanguage;
        if (language == "XML") {
            // use the primary language, not the sublanguage
            language = view.document.language
        }
    }

    // Get the help command appropriate for that language.
    var command=null, name=null;
    if (language) {
        if (gPrefs.hasStringPref(language+'HelpCommand')) {
            command = gPrefs.getStringPref(language+'HelpCommand');
        } else {
            // try to get from the language service
            var langRegistrySvc = Components.classes['@activestate.com/koLanguageRegistryService;1'].
                              getService(Components.interfaces.koILanguageRegistryService);
            var languageObj = langRegistrySvc.getLanguage(language);
            if (languageObj.searchURL) {



                command = "%(browser) "+languageObj.searchURL;

            }
        }
        if (command) {
            name = language + " Help";
        }
    }
    if (!command) {
        // Couldn't find language-specific help command -- use the default one.
        command = gPrefs.getStringPref('DefaultHelpCommand');
        name = "Help";
    }

    Run_RunCommand(window,
                   command,
                   null, // cwd
                   null, // env
                   false, // insertOutput
                   false, // operateOnSelection
                   true, // doNotOpenOutputWindow
                   'no-console', // runIn
                   0, // parseOutput
                   '', // parseRegex
                   0, // showParsedOutputList
                   name); // name
}


function launch_AlternateLanguageHelp() {
    var command = gPrefs.getStringPref('OtherHelpCommand');
    Run_RunCommand(window,
                   command,
                   null, // cwd
                   null, // env
                   false, // insertOutput
                   false, // operateOnSelection
                   true, // doNotOpenOutputWindow
                   'no-console', // runIn
                   0, // parseOutput
                   '', // parseRegex
                   0, // showParsedOutputList
                   "Alternate Help"); // name
}


function launch_Find(searchTerm) {
    _launch_FindTab("find", searchTerm);
}

function launch_Replace() {
    _launch_FindTab("replace", null);
}

var gFindDialogPanel = null;
var gFindSearchTerm = null;
function _launch_FindTab(panel, searchTerm) {
    // Transfer focus to the hidden input buffer to capture keystrokes from the
    // user while find.xul is loading. The find dialog will retrieve these
    // contents when it is ready.
    InputBuffer_Start();
    gFindSearchTerm = searchTerm;
    gFindDialogPanel = panel; // special global to pass info to find.xul
    window.openDialog("chrome://komodo/content/find/find.xul",
                      "komodo_find",
                      "chrome,close=yes");
}

function launch_FindInFiles(searchTerm, folders, fileTypes) {
    _launch_FindInFilesTab("find", searchTerm, folders, fileTypes);
}

var gFindInFilesCwd = null;
var gFindInFilesSearchTerm = null;
var gFindInFilesFolders = null;
var gFindInFilesFiletypes = null;
function _launch_FindInFilesTab(panel, searchTerm, folders, fileTypes) {
    // Transfer focus to the hidden input buffer to capture keystrokes from
    // the user while findInFiles.xul is loading. The find dialog will
    // retrieve these contents when it is ready.
    InputBuffer_Start();

    gFindInFilesSearchTerm = searchTerm;
    gFindInFilesFolders = folders;
    gFindInFilesFiletypes = fileTypes;
    var view = gViewMgr.currentView;
    if (view != null &&
        view.getAttribute("type") == "editor" &&
        view.document.file &&
        view.document.file.isLocal) {
        gFindInFilesCwd = view.document.file.dirName;
    } else {
        gFindInFilesCwd = null;
    }
    window.openDialog("chrome://komodo/content/find/findInFiles.xul",
                      "komodo_find_in_files",
                      "chrome,close=yes");
}


function launch_RunCommand() {
    // Transfer focus to the hidden input buffer to capture keystrokes from the
    // user while run.xul is loading.  (Get the current view before calling
    // inputbuffer start so we have the correct focus coming out of the
    // dialog.)
    InputBuffer_Start();
    window.openDialog("chrome://komodo/content/run/run.xul",
                      "komodo_run",
                      "chrome,close=yes");
}


function launch_DiffWindow(diff, title /* ="Diff" */, message /* =null */)
{
    if (typeof(title) == "undefined") {
        title = "Diff";
    }
    if (typeof(message) == "undefined") {
        message = null;
    }

    var obj = new Object();
    obj.title = title;
    obj.diff = diff;
    obj.message = message;
    openWindowMultipleInstance(
        "chrome://komodo/content/dialogs/diff.xul",
        "komodo_diff_window",
        defaultOpenOptions + ",resizable=yes,scrollbars=yes",
        obj);
}


function launch_watchLocalFile() {
    var filename = filepicker_openFile();
    if (filename)
        openWindowMultipleInstance("chrome://komodo/content/tail/tail.xul",
                                            "komodo_tail",
                                            defaultOpenOptions + ",resizable,scrollbars",
                                            filename,
                                            window);
}



























function launch_openAddonsMgr()
{
    const EMTYPE = "Extension:Manager";
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                       .getService(Components.interfaces.nsIWindowMediator);
    var theEM = wm.getMostRecentWindow(EMTYPE);
    if (theEM) {
        theEM.focus();
        return;
    }
  
    const EMURL = "chrome://mozapps/content/extensions/extensions.xul";
    const EMFEATURES = "chrome,menubar,extra-chrome,toolbar,resizable";
    window.openDialog(EMURL, "", EMFEATURES);
}

//---- Input buffering
// When you need to capture user input while a slow XUL window is loading you
// can use the input buffer. Usage:
//  - in some JS code:
//      InputBuffer_Start()
//      // open XUL window
//
//  - in slow XUL window onload handler:
//      var contents = InputBuffer_Finish();
//      // use the contents somehow
//

var _gInputBuffer_active;

function InputBuffer_Start()
{
    _gInputBuffer_active = 1;
    var inputBufferWidget = document.getElementById("hidden-input-buffer");
    inputBufferWidget.focus();
}

function InputBuffer_OnFocus(event)
{
    // if it is not active the hidden input buffer should not have the focus
    if (!_gInputBuffer_active && gViewMgr.currentView) {
        // This has to be in a timeout for the controllers to work right.
        window.setTimeout('gViewMgr.currentView.setFocus();', 1)
    }
}


function InputBuffer_Finish()
{
    // Return the contents of the input buffer and stop buffering.
    var inputBufferWidget = document.getElementById("hidden-input-buffer");
    var contents = inputBufferWidget.value;
    inputBufferWidget.value = "";

    _gInputBuffer_active = 0;
    return contents;
}


