/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/* Toolbox Manager
 *
 * The Toolbox manager reads items from a "per-user" project file called
 * "toolbox.kpf" in the user's userDataDir. There is also a possible
 * "Shared Toolbox" (and associated manager) in Komodo IDE builds.
 *
 * The Toolboxes are heavily modeled on the Project Manager.
 */
// Dev Notes:
// - This module would better be named toolbox.js and that prefix should
//   then be used more regularly in here.

//---- globals

var toolboxMgr = null; //XXX should be gToolboxMgr





var gToolboxCtrl = null;

//---- Public generic Toolbox methods

function Toolbox_onLoad()
{
    toolboxMgr = new toolboxManager();
    toolboxMgr.init();






    gToolboxCtrl = new toolbox_toolboxController();
    window.controllers.appendController(gToolboxCtrl);
}


function Toolbox_Export() {
    var filename = filepicker_saveFile(
            null, null, // default dir and filename
            "Export To...", // title
            "Komodo Project", // default filter
            ["Komodo Project", "All"]); // filters
            // When have .ktf files changes to this:
            //"Komodo Toolbox", // default filter
            //["Komodo Toolbox", "Komodo Project", "All"]);
    if (filename == null)
        return;

    var koDirs = Components.classes["@activestate.com/koDirs;1"].
         getService(Components.interfaces.koIDirs);
    var shutil = Components.classes["@activestate.com/koShUtil;1"].
        getService(Components.interfaces.koIShUtil);

    var userDataDir = koDirs.userDataDir;
    var os = Components.classes["@activestate.com/koOs;1"].
        getService(Components.interfaces.koIOs);
    var toolboxfname = os.path.join(userDataDir, 'toolbox.kpf');
    try {
        shutil.copyfile(toolboxfname, filename);
    } catch (e) {
        // Ignore it -- we can just go along w/ an empty toolbox (see below).
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
            getService(Components.interfaces.koILastErrorService);
        alert("Could not make a copy of the toolbox data file: " +
              lastErrorSvc.getLastErrorMessage());
    }
}

function Toolbox_ExportItems(items) {
    var defaultfilename = "exported_toolbox.kpf";
    var filename = filepicker_saveFile(
            null, defaultfilename, // default dir and filename
            "Export Toolbox To...", // title
            "Komodo Project", // default filter
            ["Komodo Project", "All"]); // filters
            // When have .ktf files changes to this:
            //"Komodo Toolbox", // default filter
            //["Komodo Toolbox", "Komodo Project", "All"]);
    if (filename == null)
        return;

    var project = Components.classes["@activestate.com/koProject;1"]
                  .createInstance(Components.interfaces.koIProject);
    try {
        var url = uriparse_localPathToURI(filename);
        project.create();
        project.prefset.setBooleanPref("import_live", false);
        project.url = url;
        for (i = 0; i < items.length; i++) {
            project.addChild(items[i]);
        }
        project.save();
    } catch(ex) {
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
            getService(Components.interfaces.koILastErrorService);
        alert('There was an error saving project "' +
              project.name + '": ' +
              lastErrorSvc.getLastErrorMessage());
    }
}

function Toolbox_ExportPackageItems(items) {
    if (typeof(items) == 'undefined' || !items || items.length < 1) {
        dialog_alert("Please select items in the toolbox to export first.");
        return;
    }
    var localPath = filepicker_saveFile(
            null, items[0].name + ".kpz", // default dir and filename
            "Export Selected Items to a Package", // title
            "Komodo Package", // default filter name
            ["All"]); // filter names to show
    if (localPath == null) {
        return;
    }
    var packager = Components.classes["@activestate.com/koProjectPackageService;1"]
                      .getService(Components.interfaces.koIProjectPackageService);
    packager.packageParts(localPath, items.length, items, true);
}

function Toolbox_ImportPackage(filename) {
    part_ImportFromPackage(toolboxMgr.toolbox, filename);
    toolboxMgr.viewMgr.view.refresh(toolboxMgr.toolbox);
}

//---- A common virtual base class for both toolbox managers (user and shared)
// Dev Notes:
// - Originally there was only the one "user" or "personal" toolbox (called
//   just the "Toolbox"). When the "Shared Toolbox" was added this base
//   class was factored out. There may be remnants of "Toolbox"-specific
//   stuff in here.
//

function toolboxBaseManager() {
    KomodoRegisterCanCloseHandler(this.save, this);
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
toolboxBaseManager.prototype = new baseManager();
toolboxBaseManager.prototype.constructor = toolboxBaseManager;

// Common init code for all toolboxes. This loads the toolbox data, creating
// and empty .kpf file if necessary. If the toolbox data is corrupt it is
// moved out of the way and a new empty one is created.
//
//  "prettyName" is a human readable name, useful for error messages and
//      debugging.
//  "elementId" is the id of the <partviewer/> XUL element for this toolbox.
//  "fname" is the full local path to the toolbox KPF file.
//
// This method returns null if successful. Otherwise it returns one of the
// following error strings:
//  "could not create": The toolbox file did not exist and an attempt to
//      create an empty starter one failed. The actual error is available
//      via koILastErrorService.
//  "could not load": The toolbox file could not be loaded for whatever
//      reason (perhaps it is corrupt?).
//
toolboxBaseManager.prototype._init = function(prettyName, elementid, fname) {
    try {
        this.lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                getService(Components.interfaces.koILastErrorService);
        var osPath = Components.classes["@activestate.com/koOsPath;1"].
                     getService(Components.interfaces.koIOsPath);

        this.prettyName = prettyName;
        this.log = getLoggingMgr().getLogger('toolboxManager: ' + elementid);
        //this.log.setLevel(LOG_DEBUG);
        this.log.info('initializing toolboxBaseManager');
        this.viewMgr = document.getElementById(elementid);
        this.viewMgr.onLoad(this);
        // XXX our one and only datapoint for the toolbox
        //this.viewMgr.view.datapoints['Name']='Name';

        this.toolbox = Components.classes["@activestate.com/koProject;1"]
                       .createInstance(Components.interfaces.koIProject);
        this.toolboxURL = uriparse_localPathToURI(fname);

        // Create an empty toolbox file if there currently isn't one.
        if (!osPath.exists(fname)) {
            this.log.info("The "+prettyName+" at '"+fname+"' does not "+
                          "exist. Creating a new empty one.");
            try {
                this.toolbox.create();
                this.toolbox.prefset.setBooleanPref("import_live", false);
                this.toolbox.url = this.toolboxURL;
                this.toolbox.save();
            } catch(ex) {
                this.toolbox = null;
                return "could not create";
            }
        }

        // Make sure that there is always a gFocusedProjectView
        gFocusedProjectView = this.viewMgr;
        gPartManagers.push(this);

        // Load the toolbox file.
        try {
            this.log.info("Loading "+prettyName+" from '"+this.toolboxURL+"'");
            this.toolbox.load(this.toolboxURL);
            this.toolbox.prefset.setBooleanPref("import_live", false);
        } catch(ex) {
            log.exception(ex);
            return "could not load";
        }

        this.loadFromProject(this.toolbox);

        // Find out if we're read-only or not.
        if (this.viewMgr.hasAttribute('lock_icon_id')) {
            var lockId = this.viewMgr.getAttribute('lock_icon_id');
            var lockIcon = document.getElementById(lockId);
            if (this.writeable()) {
                lockIcon.removeAttribute('readonly');
            } else {
                lockIcon.setAttribute('readonly',"chrome://komodo/skin/images/small_lock.png");
            }
        }

        // let the status service know it has work to do
        try {
            gObserverSvc.notifyObservers(this, 'file_update_now', null);
        } catch(e) { /* exception if no listeners */ }

        return null;
    } catch (e) {
        log.exception(e);
    }
    return "An unknown exception occurred";
}

toolboxBaseManager.prototype.close = function(sortdir) {
    try {
        if (this.toolbox && !this.toolbox.getFile().isReadOnly) {
            this.save();
            this.toolbox.close();
        }
    } catch (e) {
        log.exception(e);
    }
}

toolboxBaseManager.prototype.writeable = function () {
    // Return true if the current toolbox is writeable
    // Note: We don't trust
    //           this.wrappedproject.properties["isReadOnly"]
    //       because it is using a bogus algorithm (see URIlib.py). Instead we
    //       do our own checking using os.access().
    /// XXX nice to know, too bad there is no further explanation in URIlib.py - smc
    var os = Components.classes["@activestate.com/koOs;1"].
             getService(Components.interfaces.koIOs);
    var fname = uriparse_URIToLocalPath(this.toolboxURL);
    return os.access(fname, os.W_OK);
}

toolboxBaseManager.prototype.removeItem = function(item, skipdialog)
{
    if (baseManager.prototype.removeItem.apply(this, [item,skipdialog])) {
        // XXX fixme, project needs to make view refresh
        this.viewMgr.view.refresh(item.parent);
        this.save();
    }
}

toolboxBaseManager.prototype.removeItems = function(items, trash) {
    baseManager.prototype.removeItems.apply(this, [items, trash]);
    this.save();
}

toolboxBaseManager.prototype.addItem = function(/* koIPart */ part, /* koIPart */ parent) {
    if (typeof(parent)=='undefined' || !parent) {
        parent = this.viewMgr.view.toolbox;
    } else {
        // throw an exception if we received an index instead of a koIPart
        try {
            parent.QueryInterface(Components.interfaces.koIPart);
        } catch(e) {
            throw new Error("AddItem API changed, see toolboxManager.js:toolboxBaseManager.prototype.addItem");
        }
    }
    var isproject = false;
    try {
        /* we don't allow projects to be added to the
           toolbox, but we do allow links to projects, which
           allows us to open projects from the toolbox */
        isproject = part.QueryInterface(Components.interfaces.koIProject) != null;
    } catch(e) {}

    if (isproject) {
        var p_part = this.toolbox.createPartFromType('ProjectRef');
        p_part.setStringAttribute('url', part.url);
        p_part.setStringAttribute('name', os.path.withoutExtension(part.getStringAttribute('name')));
        part = p_part;
    }

    try {
        // XXX FIXME, I'm still not happy with this
        // if the parent is not a container, find a container
        try {
            parent.QueryInterface(Components.interfaces.koIContainerBase);
        } catch(e) {
            parent = parent.parent;
        }
        parent.addChild(part);
        this.viewMgr.view.refresh(part.parent);
        this.viewMgr.view.selection.select(this.viewMgr.view.getIndexByPart(part));

        // this will prevent projects from being removed from project pane
        // if the project is dragged into the toolbox
        this.save();
        return !isproject;
    } catch(e) { this.log.error('toolboxBaseManager addItem exception: '+e+'\n'); }
    return false;
}

toolboxBaseManager.prototype.getItemsByURL = function(url) {
    var items = [];
    var item = this.findItemByURL(this.toolbox, null, url);
    if (item != null) items.push(item);
    return items;
}

toolboxBaseManager.prototype.findItemByURL = function (parent, type, url) {
    var child = parent.getChildWithTypeAndStringAttribute(type, "url", url, true);
    if (child) return child;
    if (parent.url == url || (parent.hasAttribute('url')
            && parent.getStringAttribute('url') == url)) {
        return parent;
    }
    return null;
}

toolboxBaseManager.prototype.isLivePath = function(url) {
    return this.toolbox.containsLiveURL(url);
}

toolboxBaseManager.prototype.getSelectedProject = function() {
    return this.toolbox;
}

toolboxBaseManager.prototype.findItemByAttributeValue= function (attribute, value) {
    return this.findChildByAttributeValue(this.toolbox, attribute, value);
}

toolboxBaseManager.prototype.findPartByTypeAttributeValue= function (type, attribute, value) {
    return this.toolbox.getChildWithTypeAndStringAttribute(type, attribute, value,true);
}

toolboxBaseManager.prototype.findPartByAttributeValue= function (attribute, value) {
    return this.toolbox.getChildByAttributeValue(attribute, value, true);
}

toolboxBaseManager.prototype.applyPartKeybindings = function () {
    this.toolbox.applyKeybindings(true);
}

toolboxBaseManager.prototype.getPartsByURL = function (url) {
    var part = this.toolbox.getChildByAttributeValue('url', url, true);
    if (part != null) return [part];
    return [];
}

toolboxBaseManager.prototype.loadFromProject = function(project) {
    gTimeline.enter('loadFromProject');
    this.viewMgr.view.toolbox = project;
    this.viewMgr.view.refresh(null);
    if (this.viewMgr.view.rowCount && this.viewMgr.view.selection) {
        this.viewMgr.view.selection.select(0);
    }
    gTimeline.leave('loadFromProject');
}

toolboxBaseManager.prototype.loadToolboxFromURL = function(url)
{
    this.toolbox = Components.classes["@activestate.com/koProject;1"]
                                .createInstance(Components.interfaces.koIProject);
    this.toolbox.load(url);
    this.toolbox.prefset.setBooleanPref("import_live", false);
    this.loadFromProject(this.toolbox);
}

toolboxBaseManager.prototype.verifyUnchanged = function() {
    if (this.toolbox && this.toolbox.getFile().hasChanged) {
        prompt = 'Your '+this.prettyName+' has changed outside Komodo, '+
                  'Saving it now will overwrite changes made outside Komodo. '+
                  'You can overwrite those changes, or load the changes which '+
                  'will overwrite the changes you have made in your toolbox.';
        var response = dialog_customButtons(prompt, ["Overwrite","Reload","Cancel"],"Cancel", null, this.prettyName+" has changed on disk");
        if (response == "Cancel") {
            return false;
        } else if (response == "Reload") {
            this.toolbox.revert();
            this.loadFromProject(this.toolbox);
            return false;
        }
    }
    return true;
}

// This is occassionally used as a Komodo "can close" handler, so it
// returns true if successful, false otherwise.
toolboxBaseManager.prototype.save = function() {
    if (this.toolbox && this.toolbox.isDirty) {
        try {
            if (!this.verifyUnchanged()) return false;
            this.toolbox.save();
            // force stat update
            // XXX BUG 56517
            return this.toolbox.getFile().hasChanged;
        } catch(ex) {
            var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                getService(Components.interfaces.koILastErrorService);
            var answer = dialog_yesNoCancel(
                "There was an error saving the "+this.prettyName+". Would " +
                    "you like to save it to another location?", // prompt
                "Yes", // default answer
                lastErrorSvc.getLastErrorMessage(), // text
                "Toolbox Error"); // title
            if (answer == "Yes") {
                while (1) {
                    var filename = filepicker_saveFile(
                            null, // default dir
                            uriparse_baseName(this.toolboxURL), // default filename
                            "Backup "+this.prettyName, // title
                            "Komodo Project", // default filter
                            ["Komodo Project", "All"]); // filters
                    if (!filename) return false;

                    this.toolbox.url = uriparse_localPathToURI(filename);
                    try {
                        this.toolbox.save();
                    } catch(ex) {
                        answer = dialog_yesNoCancel(
                            "There was an error saving the "+this.prettyName+
                                "to '"+filename+"'. Would you like to save "+
                                "it to different location?", // prompt
                            "Yes", // default answer
                            lastErrorSvc.getLastErrorMessage(), // text
                            "Toolbox Error"); // title
                        if (answer == "Yes") {
                            continue;
                        } else if (answer == "No") {
                            // do nothing
                        } else /* answer == "Cancel" */ {
                            return false;
                        }
                    }
                    break;
                }
            } else if (answer == "No") {
                // do nothing
            } else /* answer == "Cancel" */ {
                return false;
            }
        }
    }
    try {
        gObserverSvc.notifyObservers(this, 'file_changed', this.toolbox.url);
    } catch(e) { /* exception if no listeners */ }
    return true;
}

toolboxBaseManager.prototype.__defineGetter__("currentProject",
function()
{
    return this.toolbox;
});

toolboxBaseManager.prototype.getCurrentProject = function() {
    return this.toolbox;
}


//---- Support for the user/personal Toolbox

function toolboxManager() {
    try {
        toolboxBaseManager.apply(this);
    } catch (e) {
        this.log.error("error creating toolboxManager: " + e);
    }
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
toolboxManager.prototype = new toolboxBaseManager();
toolboxManager.prototype.constructor = toolboxManager;

toolboxManager.prototype.init = function() {
    try {
        // Find out what our fname should be
        var koDirs = Components.classes["@activestate.com/koDirs;1"].
                getService(Components.interfaces.koIDirs);
        var os = Components.classes["@activestate.com/koOs;1"].
                getService(Components.interfaces.koIOs);
        var userDataDir = koDirs.userDataDir;
        var fname = os.path.join(userDataDir, 'toolbox.kpf');

        // Do the init
        this.name = "PersonalToolbox";
        var err = toolboxBaseManager.prototype._init.apply(this,
                    ["Toolbox", 'toolboxview', fname]);
        if (err == "could not create") {
            var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                               getService(Components.interfaces.koILastErrorService);
            dialog_internalError("Could not create a blank toolbox file.",
                                 lastErrorSvc.getLastErrorMessage());
        } else if (err == "could not load") {
            // create a backup and start with a blank one
            var badfname = fname+".error.txt";
            this.log.error("Unable to load '"+fname+"', "+
                           "a backup will be made at '"+badfname+"'.");
            alert("There was an error loading the Toolbox from '"+fname+"'. "+
                  "A backup will be made at '"+badfname+"' and a new "+
                  "toolbox will be created.");
            try {
                var shutil = Components.classes["@activestate.com/koShUtil;1"]
                                       .getService(Components.interfaces.koIShUtil);
                shutil.copyfile(fname, badfname);
            } catch (ex) {
                // Ignore it -- we can just go along w/ an empty toolbox (see below).
                alert("Could not backup toolbox data file: " +
                      this.lastErrorSvc.getLastErrorMessage());
            }
        }

        // Install sample toolbox for a particular version if necessary
        var sampletoolbox = os.path.join(koDirs.supportDir, 'samples');
        sampletoolbox = os.path.join(sampletoolbox, 'toolbox.kpf');
        var infoSvc = Components.classes["@activestate.com/koInfoService;1"].
            getService(Components.interfaces.koIInfoService);
        var prefName = "haveInstalledSampleToolbox-Komodo" + infoSvc.version;
        if (! gPrefs.hasBooleanPref(prefName) || ! gPrefs.getBooleanPref(prefName)) {
            this.log.info("Installing sample toolbox for version " + infoSvc.version);
            this.installSamples(sampletoolbox, infoSvc.version);
            gPrefs.setBooleanPref(prefName, "true");
        }
        this.loadFromProject(this.toolbox);
        // let the status service know it has work to do
        
        var _partSvc = Components.classes["@activestate.com/koPartService;1"]
                .getService(Components.interfaces.koIPartService);
        _partSvc.toolbox = this.toolbox;

        try {
            gObserverSvc.notifyObservers(this, 'file_update_now', null);
        } catch(e) { /* exception if no listeners */ }
    } catch (e) {
        log.exception(e);
    }
}

toolboxManager.prototype.installSamples = function(sampleToolboxPath, version) {
    gTimeline.enter('toolboxManager.installSamples');
    var sample_project = Components.classes["@activestate.com/koProject;1"]
                                .createInstance(Components.interfaces.koIProject);
    sample_project.prefset.setBooleanPref("import_live", false);
    var folder = sample_project.createPartFromType('folder');

    sample_project.load(uriparse_localPathToURI(sampleToolboxPath));
    folder.name = "Samples (" + version + ")";
    this.log.info("installing samples: " + folder.name );
    var i;
    var children = new Array();
    sample_project.getChildren(children, new Object());
    children = children.value;
    for (i = 0; i < children.length; i++) {
        this.log.info("adding child " + children[i]);
        folder.addChild(children[i].clone());
    }
    this.toolbox.addChild(folder);
    gTimeline.leave('toolboxManager.installSamples');
}


//---- Support for the Shared Toolbox
//
// In a discussion btwn DavidA and TrentM it was decided that the shared
// toolbox behaviour should be the following (in light of the possibility
// that the current Komodo user might not have permissions to write to
// the shared toolbox location):
//  - If the useSharedToolbox pref is false, then shared toolbox support is
//    off.
//  - Else if the shared toolbox file exists then shared toolbox support is
//    on.
//  - Else if the user DOES have access to write to the shared toolbox file
//    location then a blank one is created (as for the normal Toolbox) and
//    shared toolbox support is on.
//  - Else (useSharedToolbox is true, the shared toolbox file does not
//    exist and the user does NOT have write access) shared toolbox support
//    is turned off.

// Update the shared toolbox state (i.e. enable or disable it).
function Toolbox_updateSharedToolbox() {





























































































}























































































function toolbox_toolboxController() {
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
toolbox_toolboxController.prototype = new Controller();
toolbox_toolboxController.prototype.constructor = toolbox_toolboxController;

toolbox_toolboxController.prototype.destructor = function() {
}

toolbox_toolboxController.prototype.is_cmd_toolboxExportPackage_enabled = function () {
    try {
        return (gFocusedProjectView && gFocusedProjectView.getSelectedItems().length > 0);
    } catch (e) {
        log.exception(e);
    }
    return false;
}

toolbox_toolboxController.prototype.do_cmd_toolboxExportPackage = function () {
    if (gFocusedProjectView) {
        Toolbox_ExportPackageItems(gFocusedProjectView.getSelectedItems());
    }
}

